import os.path
import datetime
import math
from ij import IJ
from ij import ImagePlus
from ij import WindowManager
from ij.io import RoiDecoder
from ij.plugin import ImageCalculator
from ij.process import ImageStatistics
import time
import csv

from edf import Tools
from edfgui import Parameters
from edfgui import ExtendedDepthOfField

'''
Run the Extended Depth of Field plugin, sweeping only a limited 
range of different parameters. Namely, we only parse the settings
pre-configured and used in the 'easy' version of the GUI.
'''

#------------------
# Global variables.
#------------------

global ic
global ground_truth

# 3D source image to project.
source_image = 'D:/Projects/DProj/DProjComparisonToolBox-02/Live-Notum-2018_03_08_cad_cic_LSM-C0T0.tif'

# In what folder are the files?
ground_truth_folder = 'D:/Projects/DProj/DProjComparisonToolBox-02/GroundTruth'

# The ROI that contains the area over which to compute the RMSE. We use this to remove 
# are of the image where there is no tissue.
roi = RoiDecoder.open( os.path.join( ground_truth_folder, 'ProjectionROI.roi' ) )

# The ground truth image.
ground_truth = IJ.openImage( os.path.join( ground_truth_folder, 'C0T0_GT_Proj.tif' ) )

# Where to save the results.
save_folder = 'D:/Projects/DProj/DProjComparisonToolBox-02/Methods/ExtendedDepthOfField/Results'

ic = ImageCalculator()

#------------------
# Functions.
#------------------

def compute_rmse( proj ):
	# Subtract ground-truth to compute error.
	diff = ic.run( "Subtract create 32-bit", ground_truth, proj )
	# Take the square to have the square error.
	diff.getProcessor().sqr()
	# Set ROI to exclude the bottom-left corner that has no cell layer.
	diff.setRoi( roi )
	# Take the mean to have the mean square error.
	stats = diff.getStatistics()
	# Close the rest
	diff.close()
	proj.close()
	# Return the root mean square error.
	return math.sqrt( stats.mean )
	


#------------------
# Parameters.
#------------------

qualities = [ s for s in range(0, 5) ]
topologies = [ s for s in range(0, 5) ]


# Results holder.
file_names = []
quality_list = []
topology_list = []
rmses = []
timing = []

print( '\n\n\n\n____________________________________' )
print( 'Starting. %s' % datetime.datetime.now() )
print( 'ExtendedDepthOfField plugin.' )

# Store results.
results_file_path = os.path.join( save_folder, 'Results.csv' )
if not os.path.exists( results_file_path ):

	print( 'Creating new resuts file.' )
	with open( results_file_path, 'w+' ) as f:
		f.write( 'ExtendedDepthOfField plugin.\n' )
		f.write( 'Started on %s\n' % datetime.datetime.now()  )
		f.write( "%s, %s, %s, %s, %s\n" % ( 'file_name', 'quality', 'topology', 'rmse', 'time_s' ) )

else:

	print( 'Found existing results file.' )
	with open( results_file_path, 'r' ) as f:
    	# Read and add to lists.
		csv_reader = csv.reader( f, delimiter=',' )
		for i in range(3):
			next( csv_reader, None )
			
		for row in csv_reader:
			file_names.append( row[ 0 ] )
			quality_list.append( int( float( row[ 3 ] ) ) )
			topology_list.append( int( float( row[ 4 ] ) ) )
			rmses.append( float( row[ 5 ] ) )
			timing.append( float( row[ 6 ] ) )

		print( 'Existing data loaded:' )
		print( "%70s  %17s  %17s  %17s  %17s\n" % ( 'file_name', 'quality', 'topology', 'rmse', 'time_s' ) )
		for r in range( len( file_names ) ):
			print( "%70s  %17s  %17s  %17s  %17s" % ( file_names[r], quality_list[r], topology_list[r], rmses[r], timing[r] ) )
		print( '\n' )

# What channel to use to create the reference surface?
channel = 0

# Load the 3D image to project.
imp = IJ.openImage( source_image )



for q in qualities:
	for s in topologies:
	
		# Create file name.
		file_name = 'ExtendedDepthOfField_quality_%02d_smoothness_%02d.tif' % ( q, s )
		if file_name in file_names:
			print( ' - Data for %s found in results file. Skipping.' % file_name )
			continue
		
		print(' - Generating ' + file_name )
		
		#------------------
		# Execute.
		#------------------

		params = Parameters()
		params.showTopology = False
		params.setTopologySettings( s )
		params.setQualitySettings( q )
		params.color = False
		nScalesAndSize = Tools.computeScaleAndPowerTwoSize( imp.getWidth(), imp.getHeight() );
		nScales = nScalesAndSize[0]
		params.maxScales = nScales
		
		edf = ExtendedDepthOfField( imp, params )
		
		# Execute StackFocuser.
		start = time.time()
		edf.process()
		local_proj = WindowManager.getCurrentImage()
		end = time.time()
		deltaT = end - start
	
		 # Compute RMSE.
		rmse = compute_rmse( local_proj )
		
		# Append results
		file_names.append( file_name )
		with open( results_file_path, 'a' ) as f:
			f.write( "%s, %s, %s, %s, %s\n" % ( file_name, q, s, rmse, deltaT ) )

print( 'Finished! - %s' % datetime.datetime.now() )
