%% Compare LocalZProjection images with 'ground truth'.


clear
close all
clc

root_folder = 'Results/SweepBinning';
regex = '(?<method>.+)_fs(?<windowSize>\d+)_ms(?<medianSize>\d+)_gauss(?<gaussSigma>.+)_binning(?<binning>\d+)_deltaZ(?<deltaZ>\d+)\.tif';

%% Ground truth.

ground_truth_img_path = '../../GroundTruth/Groundth truth by MIP.tif';
G = double( imread(ground_truth_img_path) );

mask_path =  '../../GroundTruth/ProjectionMask.tif';
M = logical ( imread( mask_path) );

%% Read timings table

timings_file_path = fullfile( root_folder, 'Timings.csv' );
timingTable = readtable( timings_file_path, 'Delimiter', ',' );


%% Test images.

files = dir( fullfile( root_folder, 'Local*.tif' ) );
file_names =  { files.name }';

% Grab analysis parameters.
tokens = regexp( file_names, regex, 'names' );

n_files = numel( file_names );

rmse            = NaN( n_files, 1 );
windowSize      = NaN( n_files, 1 );
medianSize      = NaN( n_files, 1 );
gaussSigma      = NaN( n_files, 1 );
binning         = NaN( n_files, 1 );
deltaZ          = NaN( n_files, 1 );
timings         = NaN( n_files, 1 );

for i = 1 : n_files
   
    % Grab parameters from file name.
    windowSize( i )     = str2double( tokens{i}.windowSize );
    medianSize( i )     = str2double( tokens{i}.medianSize );
    gaussSigma( i )     = str2double( tokens{i}.gaussSigma );
    binning( i )        = str2double( tokens{i}.binning );
    deltaZ( i )         = str2double( tokens{i}.deltaZ );
    
    % Lookup timing from table.
    file_name = file_names{ i };
    index = find( strcmp( file_name, timingTable.file_name), 1 );
    if ~isempty( index )
        timings( i ) = timingTable.time_s( index );
        
    end
    

    % Compute RMSE.
    I = double( imread( fullfile( root_folder, file_names{i} ) ) );
    rmse( i ) = sqrt( immse( G(M), I(M) ) );

end



%% Put results in a table.

rmseTable = table();
rmseTable.WindowSize    = windowSize;
rmseTable.MedianSize    = medianSize;
rmseTable.GaussSigma    = gaussSigma;
rmseTable.Binning       = binning;
rmseTable.DeltaZ        = deltaZ;
rmseTable.Timing        = timings;
rmseTable.RMSE          = rmse;

rmseTable = sortrows(rmseTable, 'RMSE');


%% Plot results.

polynome_degree = 7;

close
figure( 'Units', 'centimeters', 'Position', [ 17 17 20 18 ] )
set(gca, 'FontSize', 24, 'FontName', 'Arial')

yyaxis left
hold on

x = rmseTable.Binning;
y1 = rmseTable.RMSE;
y2 = rmseTable.Timing;

hs = scatter( x, y1, 80, 'o', 'Filled' );
colors = get( hs, 'CData' );

A = [ x y1 y2 ];
A = sortrows( A );
x = A(:,1);
y1 = A(:,2);
y2 = A(:,3);

% plot( x, y1, ...
%     'LineWidth', 0.5 );


p = polyfit( x, y1 , polynome_degree );
upx = linspace( min(x), max(x), 1000 );
v = polyval( p, upx );
plot( upx, v, ...
    'LineWidth', 0.5, ...
    'DisplayName', 'Polynomial fit');

xlabel('Binning parameter')
ylabel('RMSE')

% Plot timings.

yyaxis right
hl = plot( x, y2, ...
    'LineWidth', 0.5 );

scatter( x, y2, 80, 'o', 'Filled', ...
    'CData', get( hl, 'Color') );
set( gca, 'YScale', "log")

% Plot optimal point.
yl = ylim();
ml1 = yl(1);
ml2 = yl(1) + (yl(2)-yl(1)) * 0.1;

[ ~, a ] = min( y1 );
bestSize = x( a ); 
bestRMSE = y1( a );
bestTiming = y2( a );

hb = line( [ bestSize bestSize ], [ ml1 ml2 ], ...
    'Color', 'k', ...
    'LineStyle', ':' );
ht = text( bestSize, ml2, ...
    { sprintf(' Optimal binning: %d', bestSize)
    sprintf(' RMSE = %.1f,\n Timing = %.1f s', bestRMSE, bestTiming ) }, ...
    'Color', 'k', ...
    'HorizontalAlignment', 'left', ...
    'VerticalAlignment', 'bottom' ,...
    'FontSize', 20 );

ylabel('Execution time (s)')
% title(sprintf( '%s tests', tokens{1}.method ) )


export_fig -nofontswap 'RMSE_Binning.pdf'
writetable( rmseTable, 'RMSE_Binning.xlsx' )
