#@ OpService ops
#@ ConvertService convert

import os.path
import datetime
import math
from ij import IJ
from ij import ImagePlus
from ij.io import RoiDecoder
from ij.plugin import ImageCalculator
from ij.process import ImageStatistics
from net.imagej import Dataset
from fr.pasteur.iah.localzprojector.process import LocalZProjectionOp
from fr.pasteur.iah.localzprojector.process import ReferenceSurfaceParameters
from fr.pasteur.iah.localzprojector.process.ReferenceSurfaceParameters import Method
from fr.pasteur.iah.localzprojector.process import ExtractSurfaceParameters
from fr.pasteur.iah.localzprojector.process.ExtractSurfaceParameters import ProjectionMethod
import time
import csv

'''
You need to run this from the Fiji script editor.
This script will generate a series of images used to assess performance
of the Local Z Projector plugin while varying parameters.
'''

#------------------
# Global variables.
#------------------


global ic
global roi
global ground_truth


#------------------
# Parameters.
#------------------

# The root folder for the analysis of the image.
root_folder = 'D:/Projects/DProj/DProjComparisonToolBox-02'

# 3D source image to project.
source_image = os.path.join( root_folder, 'Live-Notum-2018_03_08_cad_cic_LSM-C0T0.tif' )

# In what folder are the ground truth  files?
ground_truth_folder = os.path.join( root_folder, 'GroundTruth' )

# The ROI that contains the area over which to compute the RMSE. We use this to remove
# are of the image where there is no tissue.
roi = RoiDecoder.open( os.path.join( ground_truth_folder, 'ProjectionROI.roi' ) )

# The ground truth image.
ground_truth = IJ.openImage( os.path.join( ground_truth_folder, 'GT_Proj.tif' ) )

# Where to save the results.
save_folder = os.path.join( root_folder, 'Methods/LocalZProjection/Results/SweepDeltaZ' )

binnings 		  = [ 4 ]  # Optimum
gaussianFilters   = [ 2 ]  # Optimum
filterWindowSizes = [ 1  ]  # Optimum
deltaZ			  = [ d for d in range(0, 21) ]
medianFilterSizes = [ 4 ]  # Optimum



ic = ImageCalculator()

#------------------
# Functions.
#------------------

def compute_rmse( local_proj ):
	# Convert ImgLib2 projection to IJ1 ImagePlus.
	proj = convert.convert( local_proj, ImagePlus )
	# Subtract ground-truth to compute error.
	diff = ic.run( "Subtract create 32-bit", ground_truth, proj )
	# Take the square to have the square error.
	diff.getProcessor().sqr()
	# Set ROI to exclude the bottom-left corner that has no cell layer.
	diff.setRoi( roi )
	# Take the mean to have the mean square error.
	stats = diff.getStatistics()
	# Close the rest
	diff.close()
	proj.close()
	# Return the root mean square error.
	return math.sqrt( stats.mean )




# Results holder.
file_names = []
b_list = []
g_list = []
w_list = []
m_list = []
d_list = []
rmses = []
timing = []

version = LocalZProjectionOp().getVersion()

print( '\n\n\n\n____________________________________' )
print( 'Starting. %s' % datetime.datetime.now() )
print( 'LocalZProjection plugin version: %s' % version )

# Store results.
results_file_path = os.path.join( save_folder, 'Results.csv' )
if not os.path.exists( results_file_path ):

	print( 'Creating new resuts file.' )
	with open( results_file_path, 'w+' ) as f:
		f.write( 'LocalZProjection v%s\n' % version  )
		f.write( 'Started on %s\n' % datetime.datetime.now()  )
		f.write( "%s, %s, %s, %s, %s, %s, %s, %s\n" % ( 'file_name', 'binning', 'gaussian_sigma', 'window_size', 'median_half_size', 'delta_z', 'rmse', 'time_s' ) )

else:

	print( 'Found existing results file.' )
	with open( results_file_path, 'r' ) as f:
    	# Read and add to lists.
		csv_reader = csv.reader( f, delimiter=',' )
		for i in range(3):
			next( csv_reader, None )

		for row in csv_reader:
			file_names.append( row[ 0 ] )
			b_list.append( int( float( row[ 1 ] ) ) )
			g_list.append( float( row[ 2 ] ) )
			w_list.append( int( float( row[ 3 ] ) ) )
			m_list.append( int( float( row[ 4 ] ) ) )
			d_list.append( int( float( row[ 5 ] ) ) )
			rmses.append( float( row[ 6 ] ) )
			timing.append( float( row[ 7 ] ) )

		print( 'Existing data loaded:' )


# What channel to use to create the reference surface?
channel = 0

# Load and convert the 3D image to project.
imp = IJ.openImage( source_image )
img = convert.convert( imp, Dataset )

for binning in binnings:
	for gs in gaussianFilters:
		for fs in filterWindowSizes:
			for ms in medianFilterSizes:
				for ds in deltaZ:

					# Create file name.
					file_name = 'LocalZProjection_fs%02d_ms%02d_gauss%4.2f_binning%1d_deltaZ%1d.tif' % ( fs, ms, gs, binning, ds )
					if file_name in file_names:
						print( ' - Data for %s found in results file. Skipping.' % file_name )
						continue

					print(' - Generating ' + file_name )

					# Reference surface parameters.
					params_ref_surface = ReferenceSurfaceParameters.create() \
									.method( Method.MAX_OF_STD ) \
									.zMin( 0 ) \
									.zMax( 100000 ) \
									.filterWindowSize( int( fs ) ) \
									.binning( int( binning ) ) \
									.gaussianPreFilter( gs ) \
									.medianPostFilterHalfSize( int( ms ) ) \
									.targetChannel( channel ) \
									.get()

					# Local projection parameters.
					params_proj = ExtractSurfaceParameters.create() \
									.zOffset( 0, 0 ) \
									.deltaZ( 0, int( ds ) ) \
									.projectionMethod( 0, ProjectionMethod.MIP ) \
									.get()


					#------------------
					# Execute.
					#------------------

					# Create op, specifying only input type and the two parameters.
					lzp_op = ops.op( LocalZProjectionOp, Dataset, params_ref_surface, params_proj )
					# Execute local Z projection.
					start = time.time()
					local_proj = lzp_op.calculate( img )
					end = time.time()
					deltaT = end - start

					 # Compute RMSE.
					rmse = compute_rmse( local_proj )

					# Append results
					file_names.append( file_name )
					with open( results_file_path, 'a' ) as f:
						f.write( "%s, %s, %s, %s, %s, %s, %s, %s\n" % ( file_name, binning, gs, fs, ms, ds, rmse, deltaT ) )

print( 'Finished! - %s' % datetime.datetime.now() )
