%% Compare PreMosa images with 'ground truth'.
% Plot for all parameter values.


clear
close all
clc

root_folder = 'Results/SweepGridSize';

%% Ground truth.

ground_truth_img_path = '../../GroundTruth/C0T0_GT_Proj.tif';
G = double( imread(ground_truth_img_path) );

mask_path =  '../../GroundTruth/ProjectionMask.tif';
M = logical ( imread( mask_path) );

%% Read timings table

timings_file_path = fullfile( root_folder, 'Timings.csv' );
timingTable = readtable( timings_file_path, 'Delimiter', ',' );


%% Test images.

files = dir( fullfile( root_folder, 'Output_*.tif' ) );
file_names =  { files.name }';

% Grab analysis parameters.
tokens = regexp( file_names, 'Output_r(?<gridsize>\d+)_d(?<distance>\d+)_t50_layer0_projection\.tif', 'names' );

n_files = numel( file_names );

rmse        = NaN( n_files, 1 );
gridsize    = NaN( n_files, 1 );
distance    = NaN( n_files, 1 );
timings     = NaN( n_files, 1 );

for i = 1 : n_files

    gridsize( i ) = str2double( tokens{i}.gridsize );
    distance( i ) = str2double( tokens{i}.distance );

    % Lookup timing from table.
    file_name = file_names{ i };
    index = find( timingTable.grid_size == gridsize( i ) & timingTable.distance == distance( i ), 1 );
    if ~isempty( index )
        timings( i ) = timingTable.time__s_( index );
    end

    I = double( imread( fullfile( root_folder, file_names{i} ) ) );
    rmse( i ) = sqrt( immse( G(M), I(M) ) );

end

%% Put results in a table.

rmseTable = table();
rmseTable.GridSize = gridsize;
rmseTable.Distance = distance;
rmseTable.RMSE = rmse;
rmseTable.Timing = timings;

rmseTable = sortrows(rmseTable, 'RMSE');
writetable( rmseTable, 'RMSE.xlsx' )

%% Plot results.

close
figure

yyaxis left
hold on

x = rmseTable.GridSize;
y1 = rmseTable.RMSE;
c = rmseTable.Distance;

uc = unique( c );
nuc = numel( uc );
for i = 1 : nuc
    index = ( c == uc( i ) );
    hs( i ) = scatter( x( index ), y1( index ), 25, 'o', 'Filled', ...
        'DisplayName', sprintf('Distance = %d', uc(i) ) ); %#ok<SAGROW>
end
p = polyfit( x, y1 , 9);
upx = linspace( min(x), max(x), 1000 );
v = polyval( p, upx );
hl = plot( upx, v, 'k', ...
    'LineWidth', 2, ...
    'DisplayName', 'Polynomial fit');

[ ~, a ] = min( v );
bestGridSize = upx( a );
yl = ylim();
hb = line( [ bestGridSize bestGridSize ], [ yl(1) yl(2)/2 ], ...
    'LineStyle', '--' );
text( bestGridSize, yl(2)/2, sprintf(' Best grid-size: %d', round( bestGridSize ) ), ...
    'Color', get( hl, 'Color' ) )


xlabel('Grid size parameter')
ylabel('RMSE')
title('PreMosa tests')

hl = legend( [ hs  hl ] );
set( hl, 'Box', 'off' )

yyaxis right
hold on

y2 = rmseTable.Timing;
c = rmseTable.Distance;

uc = unique( c );
nuc = numel( uc );
for i = 1 : nuc
    index = ( c == uc( i ) );
    scatter( x( index ), y2( index ), 25, 'o', 'Filled', ...
        'DisplayName', sprintf('Distance = %d', uc(i) ) );
end
p = polyfit( x, y2 , 9);
upx = linspace( min(x), max(x), 1000 );
v = polyval( p, upx );
hl = plot( upx, v, 'k', ...
    'LineWidth', 2, ...
    'DisplayName', 'Polynomial fit');


export_fig 'RMSE.pdf'
