%% Compare PreMosa images with 'ground truth'.
% We analyze and plot a sweep on the GridSize parameter, for the best value
% of the distance parameter.

clear
close all
clc

root_folder = 'Results/SweepBrightPixels';

% Found by analyzing all images.
best_distance = 1;

%% Ground truth.

ground_truth_img_path = '../../GroundTruth/C0T0_GT_Proj.tif';
G = double( imread(ground_truth_img_path) );

mask_path =  '../../GroundTruth/ProjectionMask.tif';
M = logical ( imread( mask_path) );

%% Read timings table

timings_file_path = fullfile( root_folder, 'Timings.csv' );
timingTable = readtable( timings_file_path, 'Delimiter', ',' );


%% Test images.

files = dir( fullfile( root_folder, 'Output_*.tif') );
file_names =  { files.name }';

% Grab analysis parameters.
tokens = regexp( file_names, 'Output_r(?<gridsize>\d+)_d(?<distance>\d+)_t(?<brightpixels>\d+)_layer0_projection\.tif', 'names' );

n_files = numel( file_names );

rmse        = NaN( n_files, 1 );
gridsize    = NaN( n_files, 1 );
distance    = NaN( n_files, 1 );
timings     = NaN( n_files, 1 );
brightpixels = NaN( n_files, 1 );

for i = 1 : n_files
   
    gridsize( i ) = str2double( tokens{i}.gridsize );
    distance( i ) = str2double( tokens{i}.distance );
    brightpixels( i ) = str2double( tokens{i}.brightpixels );
        
    % Lookup timing from table.
    file_name = file_names{ i };
    index = find( timingTable.grid_size == gridsize( i ) & timingTable.distance == distance( i ) & timingTable.bright_pixels == brightpixels( i ), 1 );
    if ~isempty( index )
        timings( i ) = timingTable.time__s_( index );        
    end

    I = double( imread( fullfile( root_folder, file_names{i} ) ) );
    rmse( i ) = sqrt( immse( G(M), I(M) ) );

end

%% Put results in a table.

rmseTable = table();
rmseTable.GridSize = gridsize;
rmseTable.Distance = distance;
rmseTable.BrightPixels = brightpixels;
rmseTable.RMSE = rmse;
rmseTable.Timing = timings;

rmseTable = sortrows(rmseTable, 'RMSE');

%% Plot results.

close
figure( 'Units', 'centimeters', 'Position', [ 17 17 20 18 ] )
set(gca, 'FontSize', 24, 'FontName', 'Arial')

yyaxis left
hold on
xlim( [ -0.5 100.5 ] )

x = rmseTable.BrightPixels;
y1 = rmseTable.RMSE;
y2 = rmseTable.Timing;

A = [ x y1 y2 ];
A = sortrows( A );

hs = plot( A(:,1), A(:,2), '.', ...
    'Marker', '.', ...
    'MarkerSize', 15, ...
    'LineStyle', '-', ...
    'LineWidth', 1 );

% p = polyfit( x, y1 , 11);
% upx = linspace( min(x), max(x), 1000 );
% v = polyval( p, upx );
% plot( upx, v, 'k', ...
%     'LineWidth', 2, ...
%     'DisplayName', 'Polynomial fit');

% Determine best parameter.

[ best_RMSE, a ] = min( y1 );
best_brightpixels = x( a );
best_timing = y2( a );

% ylim( [ 1e-4 1.5e-4 ] )
yl = ylim();
ml1 = yl(1);
ml2 = yl(1) + (yl(2)-yl(1)) * 0.85;

hb = line( [ best_brightpixels best_brightpixels ], [ ml1 ml2 ], ...
    'LineStyle', ':', ...
    'Color', 'k' );
text( best_brightpixels, ml2, ...
     { sprintf('Optimal bright-pixel value: %d', best_brightpixels)
    sprintf('RMSE = %.1f,\nTiming = %.1f s', best_RMSE, best_timing ) }, ...
    'Color', 'k', ...
    'FontSize', 20 )

xlabel('Bright-pixels parameter (%)')
ylabel('RMSE')



% Plot timings.

yyaxis right
hold on

y2 = rmseTable.Timing;
valid = ~isnan( y2 );

hs = plot( A(:,1), A(:,3), '.', ...
    'Marker', '.', ...
    'MarkerSize', 15, ...
    'LineStyle', 'none', ...
    'LineWidth', 0.5 );

p = polyfit( x(valid), y2(valid) , 1);
upx = linspace( min(x), max(x), 1000 );
v = polyval( p, upx );
% hl = plot( upx, v,  ...
%     'LineWidth', 0.5, ...
%     'DisplayName', 'Polynomial fit', ...
%     'Color', get(hs, 'Color') );

ylabel('Execution time (s)')
ylim( [ 3 4.5 ] )

export_fig 'RMSE_BrightPixels.pdf'
writetable( rmseTable, 'RMSE_BrightPixels.xlsx' )