%% Compare StackFocuser results with 'ground truth'.
% We analyze and plot a sweep on the filter-size parameter.

clear
close all
clc

root_folder = 'Results/';


%% Ground truth.

ground_truth_img_path = '../../GroundTruth/Groundth truth by MIP.tif';
G = double( imread(ground_truth_img_path) );

mask_path =  '../../GroundTruth/ProjectionMask.tif';
M = logical ( imread( mask_path) );

%% Read timings table

timings_file_path = fullfile( root_folder, 'Timings.csv' );
timingTable = readtable( timings_file_path, 'Delimiter', ',' );


%% Test images.

files = dir( fullfile( root_folder, 'StackFocuser_*.tif') );
file_names =  { files.name }';

% Grab analysis parameters.
tokens = regexp( file_names, 'StackFocuser_fs(?<filtersize>\d+)\.tif', 'names' );
n_files = numel( file_names );

rmse        = NaN( n_files, 1 );
filtersize  = NaN( n_files, 1 );
timings     = NaN( n_files, 1 );

for i = 1 : n_files
   
    filtersize( i ) = str2double( tokens{i}.filtersize );
        
    % Lookup timing from table.
    file_name = file_names{ i };
    index = find( strcmp( timingTable.file_name, file_name ), 1 );
    if ~isempty( index )
        timings( i ) = timingTable.time_s( index );        
    end

    I = double( imread( fullfile( root_folder, file_names{i} ) ) );
    rmse( i ) = sqrt( immse( G(M), I(M) ) );

end

%% Put results in a table.

rmseTable = table();
rmseTable.FilterSize = filtersize;
rmseTable.RMSE = rmse;
rmseTable.Timing = timings;

rmseTable = sortrows(rmseTable, 'RMSE');

%% Plot results.

close
figure( 'Units', 'centimeters', 'Position', [ 17 17 20 18 ] )
set(gca, 'FontSize', 24, 'FontName', 'Arial')

yyaxis left
hold on
xlim( [ -0.5 100.5 ] )

x = rmseTable.FilterSize;
y1 = rmseTable.RMSE;
y2 = rmseTable.Timing;

hs = scatter( x, y1, 50, 'o', 'Filled' );
p = polyfit( x, y1, 6 );
upx = linspace( min(x), max(x), 1000 );
v = polyval( p, upx );
plot( upx, v, 'k', ...
    'LineWidth', 0.5, ...
    'DisplayName', 'Polynomial fit', ...
    'Color', get(hs, 'CData'));

% Determine best parameter.

[ best_RMSE, a ] = min( y1 );
best_parameter = x( a );
best_timing = y2( a );

yl = ylim();
ml1 = yl(1);
ml2 = yl(1) + (yl(2)-yl(1)) * 0.75;

hb = line( [ best_parameter best_parameter ], [ ml1 ml2 ], ...
    'LineStyle', ':', ...
    'Color', 'k' );
text( best_parameter, ml2, ...
     { sprintf('Optimal filter-size value: %d', best_parameter)
    sprintf('RMSE = %.1f,\nTiming = %.1f s', best_RMSE, best_timing ) }, ...
    'Color', 'k', ...
    'HorizontalAlignment', 'left', ...
    'VerticalAlignment', 'middle', ...
    'FontSize', 20 )

xlabel('Filter-size parameter')
ylabel('RMSE')


% Plot timings.

yyaxis right
hold on

y2 = rmseTable.Timing;
valid = ~isnan( y2 );

hs = scatter( x, y2, 50, 'o', 'Filled' );
p = polyfit( x(valid), y2(valid) , 4);
upx = linspace( min(x), max(x), 1000 );
v = polyval( p, upx );
hl = plot( upx, v, 'k', ...
    'LineWidth', 0.5, ...
    'DisplayName', 'Polynomial fit', ...
    'Color', get(hs, 'CData') );

ylabel('Execution time (s)')

export_fig 'RMSE_FilterSize.pdf'
writetable( rmseTable, 'RMSE_FilterSize.xlsx' )