%% Compare segmentation results with ground truth.

close all
clear
clc

% Segmentation results must be stored as an 8-bit tif file, with the cells
% pixel values > 0 and the contour between cells = 0, as the output of the
% watershed algorithm with connectivity = 4.

% Save results?
do_save = true;

% Ground truth segmentation image
gt_path = 'GT.tif';

% Methods to iterate

methods = {
    'GT'
    'MIP'
    'LocalZProjector'
    'StackFocuser'
    'SurfCut'
    'PreMosa'
    'EDF'
    'MinCostZSurface'
    'FastSME'
    'LocalZProjector-Good'
    };
n_methods = numel( methods );


oces = NaN( n_methods, 1 );

for i = 1 : n_methods
    
    method = methods{ i };
    fprintf( '\nMETHOD: %s\n', method )
    
    % Segmentation results of tested method.
    % Must be a 8-bit binary image with try pixel = 255, false = 0 (à la
    % ImageJ).
    test_path = sprintf( 'Segmentation_of_%s.tif', method );
    
    % BW image.
    GT = imread( gt_path );
    I = imread( test_path );
    
    % Mask.
    mask_path =  '../GroundTruth/ProjectionMask.tif';
    M = logical ( imread( mask_path) );
    GT( ~M ) = 0;
    I( ~M ) = 0;
    
    
    % Connectivity matrices with connectivity = 4.
    GTcc = bwconncomp(GT ,4 );
    Icc = bwconncomp(I ,4 );
    
    fprintf('Found %d cells in ground-truth image.\n',  GTcc.NumObjects )
    fprintf('Found %d cells in test image.\n', Icc.NumObjects )
    
    %% Object-level consistency error.
    % We use the same notations than in Polak et al., 2009.
    
    [ oce, CE1, CE2 ] = polakoce( GTcc, Icc, true );
    oces( i ) = oce;
    
    
    %% Output error maps.
    
    EM1 = create_error_map( GTcc, CE1 );
    hf1 = figure;
    imshow( EM1, [0 1] )
    colormap( hf1, hot )
    set(gcf, 'Name', sprintf('GT vs %s', method ) )
    
    EM2 = create_error_map( Icc, CE2 );
    hf2 = figure;
    imshow( EM2, [0 1] )
    colormap( hf2, hot )
    set(gcf, 'Name', sprintf('%s vs GT', method ) )
    
    %% Save all.
    
    if do_save
        clear hf1 hf2 %#ok<UNRCH>
        save( sprintf('Segmentation_GT_vs_%s.mat', method) )
    end
    
    
    
end


%% Functions.

function EM = create_error_map( CC, error )
    EM = NaN( CC.ImageSize, 'single' );
    for j = 1 : CC.NumObjects
        EM( CC.PixelIdxList{ j } ) = error( j );
    end
end


