function [ oce, CE1, CE2 ] = polakoce( GTcc, Icc, verbose )
% POLAKOCE Object-level consistency error metric for image segmentation.
% 
%   OCE = POLAKOCE( GT, I ) implements the object-level consistency error
%   from Polak, Mark, Hong Zhang, and Minghong Pi. 
%   "An evaluation metric for image segmentation of multiple objects." 
%   Image and Vision Computing 27.8 (2009): 1223-1227. 
%
%   The object-level consistency error (OCE) is an error metric for
%   segmentation images containing many segmented objects. It returns a
%   number between 0 and 1, rating the accuracy of a segmentation stored
%   in Icc, against a ground-truth segmentation stored in GTcc.
%
%   GTcc and Icc must be the connected components structure CC returned by
%   bwconncomp.
%
%   [ OCE, ERROR_GS, ERROR_SG ] = POLAKOCE( GT, I ) also returns the
%   partial error measure of the test image vs the ground truth, and of the
%   ground truth vs the test image. They are returned as a column vector,
%   one value object in the ground truth and test image, with the same
%   index than the object index in the 'PixelIdxList' field of the CC
%   structure.
%
%   It takes quite a while to compute. For 3000 x 3000 objects it takes
%   about 12 minutes.
%   
%   See also BWCONNCOMP, JACCARD, DICE.

%   Copyright 2019 Jean-Yves Tinevez, Institut Pasteur.

%% Inputs.

if nargout < 3
    verbose = false;
end


%% Main.

% First, Egs
if verbose
    tic
    fprintf('Step 1: ' )
end
[ Egs, CE1 ] = partial_error_measure( GTcc, Icc );

% Then the symetric.
if verbose
    fprintf('Step 2: ' )
end
[ Esg, CE2 ] = partial_error_measure( Icc, GTcc );

% Final results.
oce = min( Egs, Esg );

if verbose
    fprintf('Object-level consistency error: %.6f\n', oce )
    toc
end



%% Functions.

    % Partial error measure
    function [ Egs, consistency_error ] = partial_error_measure( GTcc, Icc )

        M = GTcc.NumObjects;
        N = Icc.NumObjects;

        Egs = 0;
        consistency_error = NaN( M, 1 );
        
        if verbose
            fprintf( '[%s]', repmat( '-', 1 , 50 ) );
        end
        
        for j = 1 : M

            if verbose
                fprintf( repmat( '\b', 1 , 52 ) )
                fprintf('[%50s]', repmat( '#', 1 , floor( 50 * j / M ) ) )
            end

            Aj = GTcc.PixelIdxList{ j };

            sum_ei = 0;
            for i = 1 : N
                Bi = Icc.PixelIdxList{ i };

                intersect_AB = ismember( Aj, Bi);
                if ~any( intersect_AB )
                    continue
                end

                union_AB = union( Aj, Bi );
                ei = cross_weight( GTcc, j, Icc, i ) * sum( intersect_AB ) / numel( union_AB );
                sum_ei = sum_ei + ei;
            end

            consistency_error( j ) = ( 1 - sum_ei );
            Egs = Egs + ( 1 - sum_ei ) * self_weight( GTcc, j );
        end
        
        if verbose
            fprintf('\n')
        end
    end

    % Self weight
    function wj = self_weight( GTcc, j )

        sum_wj = 0;
        M = GTcc.NumObjects;
        for l = 1 : M
            Al = GTcc.PixelIdxList{ l };
            sum_wj = sum_wj + numel( Al );
        end

        Aj = GTcc.PixelIdxList{ j };
        wj = numel( Aj ) / sum_wj;
    end

    % Crossed weights.
    function wji = cross_weight( GTcc, j, Icc, i )

        Bi = Icc.PixelIdxList{ i };
        Aj = GTcc.PixelIdxList{ j };
        lab = ismember( Aj, Bi);
        if ~any( lab )
            wji = 0;
            return
        end

        % Summed cross weights.
        sum_wji = 0;

        N = Icc.NumObjects;
        for k = 1 : N
            Bk = Icc.PixelIdxList{ k };
            lab = ismember( Aj, Bk);
            if ~any(lab)
                continue
            end
            sum_wji = sum_wji + numel( Bk );
        end

        % This crossed weight.
        wji = numel( Bi ) / sum_wji;
    end


end